@@ -3,6 +3,14 @@
  */
 function StartFarabi(editorId) {
 
+	// Cache dialog jQuery references for later use
+	var $actions_dialog     = $("#actions_dialog");
+	var $output_dialog      = $("#output_dialog");
+	var $options_dialog     = $("#options_dialog");
+	var $help_dialog        = $("#help_dialog");
+	var $help_search_dialog = $("#help_search_dialog");
+	var $perl_critic_dialog = $("#perl_critic_dialog");
+
 	function winHeight() {
 		return window.innerHeight || (document.documentElement || document.body).clientHeight;
 	}
@@ -89,6 +97,26 @@ function StartFarabi(editorId) {
 					humane.log("No selection found!");
 				}
 			},
+			'Ctrl-1': function(cm) {
+				humane.log("Actions that you can do with Farabi");
+				$("div").dialog("close");
+				$actions_dialog.dialog("open");
+			},
+			'Ctrl-2': function(cm) {
+				humane.log("Output stores stuff that needs to be reviewed");
+				$("div").dialog("close");
+				$output_dialog.dialog("open");
+			},
+			'Ctrl-3': function(cm) {
+				humane.log("Options provides a way to configure Farabi");
+				$("div").dialog("close");
+				$options_dialog.dialog("open");
+			},
+			'Ctrl-4': function(cm) {
+				humane.log("General help about Farabi");
+				$("div").dialog("close");
+				$help_dialog.dialog("open");
+			},
 		},
 		onCursorActivity: function() {
 			// Highlight active line
@@ -110,19 +138,15 @@ function StartFarabi(editorId) {
 	// Editor is by default Perl
 	changeMode(editor, 'perl');
 
+	// focus!
+	editor.focus();
+
 	// Farabi should be fullscreen
 	goFullScreen(editor);
 
 	// Highlight active line
 	var hlLine = editor.setLineClass(0, "activeline");
 
-	// Cache dialog jQuery references for later use
-	var $actions_dialog = $("#actions_dialog");
-	var $output_dialog = $("#output_dialog");
-	var $options_dialog = $("#options_dialog");
-	var $help_dialog = $("#help_dialog");
-	var $help_search_dialog = $("#help_search_dialog");
-
 	$("#run_in_browser_button").click(function() {
 		$actions_dialog.dialog("close");
 		$output_dialog.dialog("open");
@@ -150,16 +174,25 @@ function StartFarabi(editorId) {
 			$actions_dialog.dialog("close");
 			if(violations.length > 0) {
 				humane.log("Found Perl::Critic violations!");
-				$output.val(violations.join('\n'));
-				$output_dialog.dialog("open");
+				var html = '';
+				for(var i = 0; i < violations.length; i++) {
+					var violation = violations[i];
+					var policy = violation.policy;
+					html += '<strong>Policy:</strong><br/><a target="_blank" href="https://metacpan.org/module/' + policy + '">' + policy + '</a>';
+					html += '<br/><strong>Line:</strong><br/>' + violation.line_number;
+					html += '<br/><strong>Description:</strong><br/>' + violation.description;
+					html += '<br/><strong>Explanation:</strong><br/>' + violation.explanation;
+					html += '<br/><strong>Diagnositcs:</strong><br/>' + violation.diagnostics;
+					html += '<br/><br/>';
+				}
+				$('.results', $perl_critic_dialog).html(html);
+				$perl_critic_dialog.dialog("open");
 			} else {
 				humane.log('Your code passed Perl::Critic!');
 			}
 		});
 	}).button();
 
-	
-
 	$("#run_button").click(function() { 
 		humane.log("Not implemented yet!");
 	}).button().hide();
@@ -177,69 +210,15 @@ function StartFarabi(editorId) {
 	}).button().hide();
 
 	$("#output_tabs").tabs();
-	$output_dialog.dialog({autoOpen: false, title: "Output", width: 500});
-	$options_dialog.dialog({autoOpen: false, title: "Options"});
-	$actions_dialog.dialog({autoOpen: false, title: "Actions"});
-	$help_dialog.dialog({autoOpen: false, title: "Help"});
-	$help_search_dialog.dialog({autoOpen: false, title: "Help Search", width: 600});
+	$output_dialog.dialog({autoOpen: false, title: "Output", width: 500, close: function(event, ui) { editor.focus(); }});
+	$options_dialog.dialog({autoOpen: false, title: "Options", close: function(event, ui) { editor.focus(); }});
+	$actions_dialog.dialog({autoOpen: false, title: "Actions", close: function(event, ui) { editor.focus(); }});
+	$help_dialog.dialog({autoOpen: false, title: "Help", close: function(event, ui) { editor.focus(); }});
+	$help_search_dialog.dialog({autoOpen: false, title: "Help Search", width: 600, close: function(event, ui) { editor.focus(); }});
+	$perl_critic_dialog.dialog({autoOpen: false, title: "Perl::Critic", width: 600, close: function(event, ui) { editor.focus(); }});
 
 	$("#js-result + .CodeMirror").css("width", "100%");
 	$("#editor + .CodeMirror").css("box-shadow", "0px 0px 81px 11px gray");
-
-	var $output = $('#output');
-
-	$("#actions_dialog_selector").mouseover(function() {
-		$(this).animate({
-			opacity: 1.0,
-		});
-		humane.log("Actions that you can do with Farabi");
-	}).mouseout(function() {
-		humane.remove();
-		$(this).animate({opacity: 0.3});
-	}).click(function() {
-		$("div").dialog("close");
-		$actions_dialog.dialog("open");
-	});
-
-	$("#output_dialog_selector").mouseover(function() {
-		$(this).animate({
-			opacity: 1.0,
-		});
-		humane.log("Output stores stuff that needs to be reviewed");
-	}).mouseout(function() {
-		humane.remove();
-		$(this).animate({opacity: 0.3});
-	}).click(function() {
-		$("div").dialog("close");
-		$output_dialog.dialog("open");
-	});
-
-	$("#options_dialog_selector").mouseover(function() {
-		$(this).animate({
-			opacity: 1.0,
-		});
-		humane.log("Options provides a way to configure Farabi");
-	}).mouseout(function() {
-		humane.remove();
-		$(this).animate({opacity: 0.3});
-	}).click(function() {
-		$("div").dialog("close");
-		$options_dialog.dialog("open");
-	});
-
-	$("#help_dialog_selector").mouseover(function() {
-		$(this).animate({
-			opacity: 1.0,
-		});
-		humane.log("General help about Farabi");
-	}).mouseout(function() {
-		humane.remove();
-		$(this).animate({opacity: 0.3});
-	}).click(function() {
-		$("div").dialog("close");
-		$help_dialog.dialog("open");
-	});
-
 }
 
 function runOnPerlito(jsResultEditor, source) {
